#ifndef WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H
#define WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H

// Wolf includes
#include "core/processor/processor_base.h"
#include "gnss/capture/capture_gnss_single_diff.h"
#include "gnss/sensor/sensor_gnss.h"

// Std includes


namespace wolf {
    
WOLF_PTR_TYPEDEFS(ProcessorGnssSingleDiff);
WOLF_STRUCT_PTR_TYPEDEFS(ProcessorParamsGnssSingleDiff);


struct ProcessorParamsGnssSingleDiff : public ProcessorParamsBase
{
    Scalar time_th;
    Scalar enu_map_init_dist_min;
    ProcessorParamsGnssSingleDiff() = default;
    ProcessorParamsGnssSingleDiff(std::string _unique_name, const paramsServer& _server):
        ProcessorParamsBase(_unique_name, _server)
    {
        time_th                 = _server.getParam<Scalar>(_unique_name + "/time_th");
        enu_map_init_dist_min   = _server.getParam<Scalar>(_unique_name + "/enu_map_init_dist_min");
    }
    std::string print()
    {
        return "\n" + ProcessorParamsBase::print()
            + "time_th: " + std::to_string(time_th) + "\n";
            + "enu_map_init_dist_min: " + std::to_string(enu_map_init_dist_min) + "\n";
    }
};
    
//class
class ProcessorGnssSingleDiff : public ProcessorBase
{
    protected:
        SensorGnssPtr sensor_gnss_ptr_;
        ProcessorParamsGnssSingleDiffPtr params_gnss_;
        CaptureGnssSingleDiffPtr last_capture_ptr_, incoming_capture_ptr_;

    public:
        ProcessorGnssSingleDiff(ProcessorParamsGnssSingleDiffPtr _params_gnss, SensorGnssPtr _sensor_gnss_ptr);
        virtual ~ProcessorGnssSingleDiff();
        virtual void configure(SensorBasePtr _sensor);

    protected:
        /** \brief process an incoming capture
         *
         * Each derived processor should implement this function. It will be called if:
         *  - A new capture arrived and triggerInCapture() returned true.
         */
        virtual void processCapture(CaptureBasePtr) override;

        /** \brief process an incoming key-frame
         *
         * The ProcessorTracker only processes incoming captures (it is not called).
         */
        virtual void processKeyFrame(FrameBasePtr _keyframe_ptr, const Scalar& _time_tolerance) override {};

        /** \brief trigger in capture
         *
         * Returns true if processCapture() should be called after the provided capture arrived.
         */
        virtual bool triggerInCapture(CaptureBasePtr) override;

        /** \brief trigger in key-frame
         *
         * The ProcessorTracker only processes incoming captures, then it returns false.
         */
        virtual bool triggerInKeyFrame(FrameBasePtr _keyframe_ptr, const Scalar& _time_tolerance) override {return false;}

        virtual bool voteForKeyFrame();


    private:
        FactorBasePtr emplaceFactor(FeatureBasePtr& ftr_ptr);

    public:
        static ProcessorBasePtr create(const std::string& _unique_name, const ProcessorParamsBasePtr _params, const SensorBasePtr sensor_ptr);

};

inline bool ProcessorGnssSingleDiff::triggerInCapture(CaptureBasePtr)
{
    return true;
}

} // namespace wolf

#endif //WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H
