/**
 * \file processor_odom_3D_yaml.cpp
 *
 *  Created on: Oct 25, 2016
 *      \author: jsola
 */

// wolf yaml
#include "core/yaml/yaml_conversion.h"

// wolf
#include "core/processor/processor_odom_3D.h"
#include "core/factory.h"

// yaml-cpp library
#include <yaml-cpp/yaml.h>

namespace wolf
{

namespace
{
static ProcessorParamsBasePtr createProcessorOdom3DParams(const std::string & _filename_dot_yaml)
{
    YAML::Node config = YAML::LoadFile(_filename_dot_yaml);

    if (config["processor type"].as<std::string>() == "ODOM 3D")
    {
        YAML::Node kf_vote = config["keyframe vote"];

        ProcessorParamsOdom3DPtr params = std::make_shared<ProcessorParamsOdom3D>();

        params->max_time_span       = kf_vote["max time span"]      .as<Scalar>();
        params->max_buff_length     = kf_vote["max buffer length"]  .as<SizeEigen  >();
        params->dist_traveled       = kf_vote["dist traveled"]      .as<Scalar>();
        params->angle_turned        = kf_vote["angle turned"]       .as<Scalar>();

        return params;
    }

    std::cout << "Bad configuration file. No processor type found." << std::endl;
    return nullptr;
}

// Register in the SensorFactory
const bool WOLF_UNUSED registered_prc_odom_3D = ProcessorParamsFactory::get().registerCreator("ODOM 3D", createProcessorOdom3DParams);

} // namespace [unnamed]

} // namespace wolf

